#!/bin/sh
# shellcheck disable=SC2016

set -e

_bashrc="/etc/bash.bashrc"
_netrc="${HOME}/.netrc"

################################################
# Helper Functions
################################################

# Params: N/A
debug_deployment_script() {
  while true; do sleep 100 && wait; done
}

################################################
# Service Configuration Functions
################################################

configure_elasticache() {
  {
    echo "alias connect='redis-cli -h ${REDIS_CLUSTER_ENDPOINT} -p 6379 --tls'"
    echo 'echo "{"level":"info","msg":"Run \"connect\" to connect with redis-cli"}"'
  } >>$_bashrc
}

configure_elasticsearch() {
  ES_PROTOCOL=${ES_PROTOCOL:-https}
  ES_HOST=${ES_HOST}
  ES_PORT=${ES_PORT:-443}
  ES_URL="${ES_PROTOCOL}://${ES_HOST}:${ES_PORT}"
  ES_USER=${ES_USER}
  ES_PASSWORD=${ES_PASSWORD}

  {
    echo "export URL=${ES_URL}"
    echo "alias healthcheck='curl -u ${ES_USER}:${ES_PASSWORD} ${ES_PROTOCOL}://${ES_HOST}:${ES_PORT}/_cluster/health?pretty'"
    echo 'echo "{"level":"info","msg":"Run \"healthcheck\" to see the current cluster health"}"'
    echo 'echo "{"level":"info","msg":"Elasticsearch URL stored as environment variable \$URL"}"'
  } >>$_bashrc

  {
    echo "machine ${ES_URL}"
    echo "login ${ES_USER}"
    echo "password ${ES_PASSWORD}"
  } >>"$_netrc"
}

configure_aurora() {
  PGPORT=${PGPORT:-5432}
  PGUSER=${PGUSER}
  PGDATABASE=${PGDATABASE}
  PGHOST=${PGHOST_READER}
  PGPASSWORD=${PGPASSWORD}

  {
    echo "export PGPASSWORD=${PGPASSWORD}"
    echo "export PGHOST=${PGHOST_READER}"
    echo "alias connect='psql --pset pager=off -h ${PGHOST_READER} -p ${PGPORT} -U ${PGUSER} -d ${PGDATABASE}'"
    echo "alias wconnect='psql --pset pager=off -h ${PGHOST_WRITER} -p ${PGPORT} -U ${PGUSER} -d ${PGDATABASE}'"
    echo 'echo "{"level":"crit","msg":"Run \"connect\" to connect with psql to the reader endpoint"}"'
    echo 'echo "{"level":"crit","msg":"Run \"wconnect\" to connect with psql to the writer endpoint"}"'
  } >>$_bashrc

  # TODO: Configure psql with .pgpass file instead of ENV vars?
  # hostname:port:database:username:password
  #  _pgpass="/home/.pgpass"
  #  echo "${POSTGRES_HOST}:${POSTGRES_PORT}:${POSTGRES_DB}:${POSTGRES_USER}:${POSTGRES_PASS}" >> $_pgpass
}

################################################
# Main()
################################################

# Setup bash
{
  echo 'echo "{"level":"crit","msg":"Activated debug pod for ${AWS_SERVICE} break-fix work"}"'
} >>$_bashrc

# Configure services
printf "{\"level\":\"info\",\"msg\":\"Configuring debug deployment for %s\"}\n" "${AWS_SERVICE}"
case $AWS_SERVICE in
aurora)
  configure_aurora
  ;;
elasticache)
  configure_elasticache
  ;;
elasticsearch)
  configure_elasticsearch
  ;;
*)
  echo '{"level":"info","msg":"No additional service configuration required"}'
  ;;
esac

# Done
echo '{"level":"info","msg":"Deployment ready. Entering into sleep loop"}'
debug_deployment_script
