#!/usr/bin/env bash
set -e

################################################
# Functions
################################################

terraform_init() {
  print_header "Terraform Init"

  run_cmd terraform init -backend-config="bucket=${tfBucketName}" -backend-config="key=${tfBucketKey}" -backend-config="region=${awsDefaultRegion}" -lock=false -plugin-dir=/terraform/.terraform/providers/ $@
}

terraform_plan() {
  print_header "Terraform Plan"
  optTfVar=""

  if [ -f ${tfVarsFile} ]; then
    optTfVar="-var-file=${tfVarsFile}"
  fi

  run_cmd terraform plan "${optTfVar}" --out plan.out --lock=false $@

  run_cmd terraform show plan.out \>plan.txt
  run_cmd terraform show -json plan.out \> terraform-show.json

  print_header "Policy Scanning"

  ## Run the Conftest policy scan for fedramp/commercial compliance
  if [ "${policyScan}" == "true" ]; then
    policy_scan
  else
    cprint "policyScan is skipped"
  fi

  # Check Terraform plan for deletion/recreation actions
  blast_radius
}

terraform_apply() {
  print_header "Terraform Apply"

  # Check for tf plan file (plan.out)
  if [ ! -f plan.out ]; then
    cprint "Error: failed to find Terraform plan.out" red
    exit 1
  else
    cprint "Found Terraform plan file"
    run_cmd terraform show plan.out
  fi

  ## apply plan.out
  run_cmd terraform apply -lock=false $@ plan.out
}

terraform_destroy() {
  print_header "Terraform destroy begins"

  optTfVar=""
  if [ -f ${tfVarsFile} ]; then
    run_cmd cat ${tfVarsFile}
    optTfVar="-var-file=${tfVarsFile}"
  fi
  run_cmd terraform destroy -auto-approve ${optTfVar} -lock=false $@
  exit_code=$?
  if [ $exit_code -ne 0 ]; then
    cprint "ERROR: Terraform destroy failed"
    cprint "Exiting... "
    exit 1
  fi
}

terraform_output() {
  print_header "Terraform Output"
  run_cmd terraform output -json \> output.json
  # cat output.json | grep -v pass
}

terraform_get() { # $1 = resourceName, $2 = exportVarName
  print_header "Terraform get: $1 ; Assigning to: $2"
  export $2=$(jq --arg resourceName "$1" '.[$resourceName].value' output.json)
}

################################################
# Policy Testing
################################################

# Checks the Terraform plan for deletion/recreation actions
# Runs a custom policy stored in https://gitlab.bit9.local/production-infrastructure/cloud-governance/policy-packs
# under the ./policies/terraform/ directory in the mosaicv2 package
blast_radius() {
  cprint "- Blast Radius check starting..."
  set +e
  if [ "${forceDeploy}x" == "truex" ]; then
    cprint "forceDeploy has been set to true. Running Conftest with exceptions enabled." 'green'
    # Create the exceptions yaml to be used as data input for policy test
    cat <<EOF >config.yaml
    config:
      forceDeploy: "true"
EOF
    # Run Conftest w/ the exceptions config yaml
    run_cmd conftest test terraform-show.json --policy ../policies/ --namespace mosaicv2 --data config.yaml
    cprint "Continuing with apply." 'green'
  else
    # If FORCEDEPLOY is not true then run Conftest w/o exceptions
    run_cmd conftest test terraform-show.json --policy ../policies/ --namespace mosaicv2
    exit_code=$?
    if [ $exit_code -ne 0 ]; then
      cprint "ERROR: This terraform plan will result in resources being destroyed or recreated" 'red'
      cprint "Exiting... please set env variable FORCEDEPLOY=true to force apply this plan" 'red'
      exit 1
    fi
  fi
  set -e
  cprint "Blast Radius check passed" 'green'
}

# Checks the Terraform plan for compliance
# Runs standard policies stored in https://gitlab.bit9.local/production-infrastructure/cloud-governance/policy-packs
policy_scan() {
  cprint "- Policy Scan starting..."
  if echo "${awsDefaultRegion}" | grep -q "gov"; then
    POLICY="govcloud" # Variable is used to determine rule exceptions for the policies if GovCloud
  else
    POLICY="commercial" # ... if Commercial
  fi

  set +e
  set -o pipefail # allow pipefail to capture exit code of conftest
  conftest test terraform-show.json --policy ../policies/ --data ../data/$POLICY/ --output json | tee opa.json
  exit_code=$?
  set +o pipefail

  # Create Logz File
  if [[ -n ${LOGZ_API_TOKEN} ]] && [ $POLICY == "commercial" ]; then
    cprint "INFO: Sending OPA test results to Logz" 'yellow'
    namespace=$(jq -r '.[0].namespace' <opa.json)

    # Counts
    successes=$(jq '.[0].successes' <opa.json)
    failures=$(jq '.[0].failures | length' <opa.json)
    warnings=$(jq '.[0].warnings | length' <opa.json)
    exceptions=$(jq '.[0].exceptions | length' <opa.json)

    # Specific message outputs
    failures_msg=$(python3 -c "import json,sys,jmespath; data=json.load(sys.stdin); path=jmespath.search('[].failures[].msg', data); print('; '.join(path).replace('\n',''))" <./opa.json | sed -r "s/\\\"/\\'/g")
    warnings_msg=$(python3 -c "import json,sys,jmespath; data=json.load(sys.stdin); path=jmespath.search('[].warnings[].msg', data); print('; '.join(path).replace('\n',''))" <./opa.json | sed -r "s/\\\"/\\'/g")
    exceptions_msg=$(python3 -c "import json,sys,jmespath; data=json.load(sys.stdin); path=jmespath.search('[].exceptions[].msg', data); print('; '.join(path).replace('\n',''))" <./opa.json | sed -r "s/\\\"/\\'/g")

    # Send to logz.io
    echo "{\"metrics\":{\"successes\":${successes:-0},\"failures\":${failures:-0},\"warnings\":${warnings:-0},\"exceptions\":${exceptions:-0}},\"message\":\"${externalNamingPrefix:-"none"}\",\"dimensions\":{\"namespace\":\"${namespace:-"none"}\", \"failures_msg\":\"${failures_msg:-"none"}\", \"warnings_msg\":\"${warnings_msg:-"none"}\", \"exceptions_msg\":\"${exceptions_msg:-"none"}\"}}" |
      curl -X POST "https://listener.logz.io:8071?token=${LOGZ_API_TOKEN}" --data-binary @-
    post_exit_code=$?
    if [ $post_exit_code -ne 0 ]; then
      cprint "ERROR: Error sending logs to Logz.io" 'red'
    fi
  fi
  set -e

  # Check exit code
  if [ $exit_code -ne 0 ]; then
    cprint "ERROR: Policy Scan failed, please check the test results and update your plan" 'red'
    exit $exit_code
  else
    cprint "Policy Scan passed" 'green'
  fi
}

################################################
# Main
################################################

print_header "Begin terraform.sh"


source /scripts/aws-discovery.sh
 
# when tfversion is empty , use 1.1.9 (default) . otherwise, use 1.3.7
if [ "${tfVersion}x" == "x" ]; then
  run_cmd tfswitch 1.1.9
else
  run_cmd tfswitch 1.3.7
fi

echo "The tf version being used for this deployment is"
terraform -v

export tfBucketName=${tfBucketNamePrefix}-${awsAccountNumber}-${awsDefaultRegion}
export tfBucketKey=${s3BucketKey}

if [ -z "${tfRoot}" ]; then
  export tfRoot=/terraform
fi

if [ -z "${policyScan}" ]; then
  cprint "Must set env policyScan" 'red'
  exit 1
fi

export tfVarsFile=/etc/config/settings.tfvars

cprint "show vars from tfvarFile"
# debug command
if [ -f ${tfVarsFile} ]; then
  run_cmd cat ${tfVarsFile}
fi

# print TF_VAR ( Note: TF_VARs are not working . we should convert then into ENV var)
run_cmd printenv \| grep TF_VAR \|\| true

# show other common vars
cprint "show common global var from inputConfig"
common_vars="vpc_id security_group_ids subnet_ids namingPrefix externalNamingPrefix s3BucketKey tfVersion policyScan forceDeploy"
for var in $(echo $common_vars | xargs -n1 | sort | xargs); do
  printf "%-30s = %s\n" $var "$(printenv $var)"
done

cd ${tfRoot}/
run_cmd pwd

# If we're running locally, we have to idempotently create the buckets that may not exist
# TODO - consider always doing this so it has a chance of bootstrapping on a different AWS account
if [ "${awsDeployment,,}" = "false" ]; then
  if ! aws ${awsEndpointArg} s3 ls s3://${tfBucketName} 2>/dev/null >/dev/null; then
    cprint "Creating S3 bucket ${tfBucketName}"
    run_cmd aws ${awsEndpointArg} s3 mb s3://${tfBucketName}
  fi
else
  if aws ${awsEndpointArg} s3 ls s3://${tfBucketName} 2>&1 | grep -q 'NoSuchBucket'; then
    cprint "Error: bucket ${tfBucketName} does not exist." red
    exit 1
  fi
fi
